#ifdef QT4
#include <QDialogButtonBox>
#include <QTabWidget>
#include <QShowEvent>
#include <QUrl>
#include <QTextDocument>
#else
#include <qstylesheet.h>
#include <qvbox.h>
#endif

#include <qlayout.h>

#include "playlistview.h"
#include "imagefx.h"
#include "mediainfodialog.h"
#include "media.h"
#include "helpers.h"
#include "coverartprovider.h"

#include "compathack.h"

#ifdef QT4
MediaInfoDialog::MediaInfoDialog(PlayListView *playList, const QString &noCoverImageFilename, QWidget *parent)
	:	QTabbedDialog(parent),
		playListView_(playList),
		noCoverImageFilename_(noCoverImageFilename)
{
	setModal(true);
#else
MediaInfoDialog::MediaInfoDialog(PlayListView *playList, const QString &noCoverImageFilename, QWidget *parent)
	:	QTabbedDialog(parent, 0, true),
		playListView_(playList),
		noCoverImageFilename_(noCoverImageFilename)
{
#endif

	setWindowTitle(tr("Media File Information"));
	setupWidgets();

	PlayListItem *item = playListView_->currentPlayListItem();
	if (item)
		setMedia(item->media());
}

void MediaInfoDialog::showEvent(QShowEvent *)
{
#ifdef QTOPIA
	showMaximized();
#endif;
}

void MediaInfoDialog::setupWidgets()
{
#ifdef QT4
	QTabWidget *tabWidget = new QTabWidget;
#endif

	QWidget *overview = new QWidget(this);
	QVBoxLayout *layout = new QVBoxLayout(overview);

#ifdef QT4
	overviewText_ = new QTextEdit(overview);
	overviewText_->setReadOnly(true);
	layout->addWidget(overviewText_);
#else
	mimeSourceFactory_ = new QMimeSourceFactory();

	overviewText_ = new QTextView(overview);
	overviewText_->setMimeSourceFactory(mimeSourceFactory_);
	layout->addWidget(overviewText_);

	QStyleSheetItem *item = overviewText_->styleSheet()->item("table");
	item->setFontSize(10);

	item = overviewText_->styleSheet()->item("h3");
	item->setFontSize(13);
#endif

#ifdef QT4
	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok);
    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));

    tabWidget->addTab(overview, tr("Overview"));

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(tabWidget);
    mainLayout->addWidget(buttonBox);
    setLayout(mainLayout);
#else
    addTab(overview, tr("Overview"));
#endif
}

void MediaInfoDialog::setMedia(Media *media)
{
	QImage image;

	if (gCoverArtProvider.loadCoverDirectly(image, media->artist(), media->album(), media->location().toString()) ||
		image.load(noCoverImageFilename_))
	{
		QSize outputSize = ImageFX::optimalFitSize(image.size(), QSize(80, 80));
#ifdef QT4
		QImage scaledImage = image.scaled(outputSize.height(), outputSize.width(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
		overviewText_->document()->addResource(QTextDocument::ImageResource, QUrl("coverart"), scaledImage);
#else
		QImage scaledImage = image.smoothScale(outputSize.height(), outputSize.width());
		mimeSourceFactory_->setImage("coverart", scaledImage);
#endif
	}

	QString genericInfo =
		"<table>"
		"<tr>"
		"  <td><img src=\"coverart\"/></td>"
		"  <td><br/>"
		"  " + media->title() + " (" + formatPlayTime(media->length()) + ")<br/>"
		"  " + media->artist() + "<br/>"
		"  " + media->album() +
		"  </td>"
		"</tr>"
		"</table>";

	QString audioInfo = "";

	if (media->isAudio() || media->isVideo())
	{
		audioInfo =
			"<tr>"
			"<td colspan=\"4\">"
			"<h3>Audio:</h3>"
			"</td>"
			"</tr>"
			"<tr>"
			"  <td align=\"right\">"
			"    <strong>Codec:</strong>"
			"  </td>"
			"  <td>"
			"  " + (media->mediaAudio()->codec.isEmpty() ? QString("Unknown") : media->mediaAudio()->codec) +
			"  </td>"
			"  <td align=\"right\">"
			"    <strong>Bitrate:</strong>"
			"  </td>"
			"  <td>"
			"  " + formatBitrate(media->mediaAudio()->bitrate) +
			"  </td>"
			"</tr>"
			"<tr>"
			"  <td align=\"right\">"
			"    <strong>Samplerate:</strong>"
			"  </td>"
			"  <td>"
			"  " + QString().number(media->mediaAudio()->sampleRate) + " Hz"
			"  </td>"
			"  <td align=\"right\">"
			"    <strong>Channels:</strong>"
			"  </td>"
			"  <td>"
			"  " + QString().number(media->mediaAudio()->channels) +
			"  </td>"
			"</tr>";
	}

	QString videoInfo = "";

	if (media->isVideo())
	{
		videoInfo =
			"<tr>"
			"<td colspan=\"4\">"
			"<h3>Video:</h3>"
			"</td>"
			"</tr>"
			"<tr>"
			"  <td align=\"right\">"
			"    <strong>Codec:</strong>"
			"  </td>"
			"  <td>"
			"  " + (media->mediaVideo()->codec.isEmpty() ? QString("Unknown") : media->mediaVideo()->codec) +
			"  </td>"
			"  <td align=\"right\">"
			"    <strong>Bitrate:</strong>"
			"  </td>"
			"  <td>"
			"  " + formatBitrate(media->mediaVideo()->bitrate) +
			"  </td>"
			"</tr>"
			"<tr>"
			"  <td align=\"right\">"
			"    <strong>FPS:</strong>"
			"  </td>"
			"  <td>"
			"  " + QString().number(media->mediaVideo()->FPS) +
			"  </td>"
			"  <td align=\"right\">"
			"    <strong>Resolution:</strong>"
			"  </td>"
			"  <td>"
			"  " + QString().sprintf("%d x %d", media->mediaVideo()->width, media->mediaVideo()->height) +
			"  </td>"
			"</tr>";
	}

	QString extendedInfo = "<table>" + audioInfo + videoInfo + "</table>";

	QString fileInfo =
		"<table>"
		"<tr>"
		"<td colspan=\"4\">"
		"<h3>File Info:</h3>"
		"</td>"
		"</tr>"
		"<tr>"
		"  <td align=\"right\">"
		"    <strong>Size:</strong>"
		"  </td>"
		"  <td>"
		"  " + formatFileSize(media->fileSize()) +
		"  </td>"
		"</tr>"
		"<tr>"
		"  <td align=\"right\">"
		"    <strong>Location:</strong>"
		"  </td>"
		"  <td>"
		"  " + media->location().toString() +
		"  </td>"
		"</tr>"
		"</table>";

	overviewText_->setText(
#ifdef QT4
		"<html><head/><body>" +
#endif
		genericInfo +
		extendedInfo +
		fileInfo
#ifdef QT4
		+ "</body></html>"
#endif
	);
}
